//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Make Time Series 2015/08/10
// Modified for using Julidan day 2016/07/01
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include "bin.c"
#include "jday.c"


// Length of the Data in Column/Line Direction
int X;
int Y;

// Year/Month/Day/Minute
int YYYY;
int MM;
int DD;
int MN;

int HH;

// Time of Picked Data
int Pick_Time[6+1];

// Picked Pixel
int Pick_Pix[2+1];


// Time of Picked Data
int Pick_Time_UTC[6+1];
int Pick_Time_Local[6+1];

// Extracted parameter
float Extract_param[3+1];


// Read Standard Latitude/Longitude Data
float *STN_LAT;
float *STN_LNG;

// Calculation Start Hour
int Start_HH;

// Calculation End Hour
int End_HH;

// File type
char ftype[512];

// Time File Type
char ttype[512];

// Directory Name to Read Original Data
char input_DIR[512];

// Directory Name to Write Time Series Data
char TimeS_DIR[512];

// Directory Name to Read Standart Latitude/Longitude Data
char SL_DIR[512];

// UTC
//double UTC = 9;

// Filename of Standard Latitude
char LATname[512];

// Filname of Standard Longitude
char LNGname[512];

// Solar Zenith Angle
static double SZA;

// TIME DIFF  & Time Type
double TIME_DIFF; // int->doble for India 5.5H
char TIME_Code[512];

//jd_UTC_PickUP
double jd_UTC_PickUP;

// Use for Calculate SZA
double a1 = 0.000075;
double a2 = 0.001868;
double a3 = 0.032077;
double a4 = 0.014615;
double a5 = 0.040849;

double b1 = 0.006918;
double b2 = 0.399912;
double b3 = 0.070257;
double b4 = 0.006758;
double b5 = 0.000907;
double b6 = 0.002697;
double b7 = 0.001480;


char wfilename[512];

void extract(double LAT, double LNG, FILE *pw);
void search_extract_pixel(double LAT, double LNG, int *EXT_Pix);
void time_check(int Year, int Month, int Day, int Hour, int Minute, double LAT, double LNG);
void shift_time(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, int *Shift_Time);
double cal_sza(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, double LAT, double LNG);

void mjd2utc(float Time, int *Pick_Time, int Year, int Month, int Day);
int get_lastday(int Year, int Month);
double date2doy(int Year, int Month, int Day, int Hour, int Minute);


void date_errorcheck();
void initialization();
void read_formatfile();
void read_standard_latlng();
void write_txt(FILE *pw);


int main(int argc, char *argv[])
{

  int I;

  // Filename to Write Time Series Data
  char wfilename[512];

  // Latitude/Longitude
  double LAT;
  double LNG;

  FILE *pw;        /* declar file pointer */

  YYYY = atoi(argv[1]);
  MM = atoi(argv[2]);
  DD = atoi(argv[3]);

  HH   = 0;
  MN   = 0;

  LAT = atof(argv[4]); //ascii->double
  LNG = atof(argv[5]); //ascii->double
  
  
    struct stat  st;
        char fname[20];
        char str1[20] = "output/";
        sprintf(fname,"%s%d%02d",str1,YYYY,MM);

        if (stat(fname, &st) != 0){
            printf("%s dosen't exist\n", fname);
            if(mkdir(fname, 0777)==0){
                printf("Succeeded to create a directory\n");
            }else{
                printf("Failed to create a directory\n");
            }
        }
        else {
                switch (st.st_mode & S_IFMT) {
                        case S_IFREG:
                                printf("%s is a file\n", fname);
                                break;
                        case S_IFDIR:
                                printf("%s is a dir\n", fname);
                                break;
                        default:
                                printf("%s isn't a file or dir\n", fname);
                        break;
                }
        }
        

  date_errorcheck();
  initialization();
  
  sprintf(wfilename, "%s%d%02d/%d%02d%02d.%s%s.ts.txt", TimeS_DIR, YYYY, MM, YYYY, MM, DD, TIME_Code, ftype);

  if ((pw = fopen(wfilename, "w")) == NULL) {
        printf("file open error!!\n");
        exit(EXIT_FAILURE);        /* error exit */
  }


  extract(LAT, LNG, pw);

  fclose(pw);
  free(STN_LAT);
  free(STN_LNG);
  return 0;


}//main


void extract(double LAT, double LNG, FILE *pw)
{

// Date of Data Extracted(UTC)
int UYYYY;
int UMM;
int UDD;
int UHH;
int UMN;

  int I,J;

  // Complement Flag
  int Complement_Flag;
  
  // Data Read
  float *DR;

  // Filename of Original Data
  char filename[512];

  // Time of Data
  float TimeD;
  
  // UTC Time
  int UTime[5+1];

  struct stat st;
  
  double jd_UTC = 0;

  // Answer
  float ANS;
 
  // Initialization
  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}

  double jd_jul_ymdhn=TransDateToJD(YYYY,MM,DD,HH,MN,0); //1.Local Time to JD(Local)
  jd_UTC= jd_jul_ymdhn - TIME_DIFF/24.0;        //  2.JD(Local) to JD(UTC)
  jd2ymdhn(jd_UTC,UTime);//  3.JD(UTC) to Calendar(UTC)

  UYYYY = UTime[1];
  UMM = UTime[2];
  UDD = UTime[3];
  UHH = UTime[4];
  UMN = UTime[5];


  search_extract_pixel(LAT, LNG, Pick_Pix);

  Complement_Flag = 0;

  int ifile=0;
  char strDate[512];
  sprintf(strDate, "%02d%02d%02d", UDD, UHH, UMN);
  int iDate = atoi(strDate);

  for(ifile=iDate;ifile<=iDate+10000;ifile++) { // iDate : DDHHMN  ex: 011530 - 021530
 
    sprintf(filename, "%s%d%02d/%d%02d%06d%s", input_DIR, UYYYY, UMM, UYYYY, UMM, ifile,ftype);
   
    if(stat(filename, &st) == 0){
       UYYYY = UYYYY;
       UMM = UMM;
       UDD =(int)(ifile/10000);
       UHH = (int)((ifile - UDD*10000)/100);
       UMN =(int)((ifile - UDD*10000 - UHH*100));

       time_check(UYYYY, UMM, UDD, UHH, UMN, LAT, LNG);
  
            if(SZA < 90){
                 bin2f(filename, X, Y, DR);    
                 ANS = (float)DR[X*(Pick_Pix[2]-1)+Pick_Pix[1]];
            }else{
                 ANS = 0;
            }//fi
  
            // Extract
            Extract_param[1] = (float)LAT;
            Extract_param[2] = (float)LNG;
            Extract_param[3] = ANS; 

            write_txt(pw); 

    }//if exist

  }//for

  free(DR);

  return;

}//extract


void search_extract_pixel(double LAT, double LNG, int *ANS_Pixel)
{

  int I,J;

  // Minimun Distance
  double MIN_DIS = 10000;

  // Distance of Column/Line Direction
  double DIS_X;
  double DIS_Y;

  // Pixel Number of Minimum Distance of Column/Line Direction
  int MIN_X = 1;
  int MIN_Y = 1;

  double DIS_Border;
  
  if(X<Y)
  {        
    DIS_Border = (STN_LNG[X*(1-1)+2]-STN_LNG[X*(1-1)+1])*1.5;
                             
  }else{
  
    DIS_Border = (STN_LAT[X*(1-1)+1]-STN_LAT[X*(2-1)+1])*1.5;
    
  }//fi

  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        DIS_X = pow(LNG - STN_LNG[X*(I-1)+J], 2);
        DIS_Y = pow(LAT - STN_LAT[X*(I-1)+J], 2);

        if( sqrt(DIS_X + DIS_Y) < MIN_DIS) 
        {
           MIN_DIS = sqrt(DIS_X + DIS_Y);
           MIN_X = J;
           MIN_Y = I;

        }//fi

    }//J
  }//I

  if(MIN_DIS > DIS_Border)
  {
     printf("%f, %f is out of range\n", LAT, LNG);
     exit(1);

  }//fi

  ANS_Pixel[1] = MIN_X;
  ANS_Pixel[2] = MIN_Y;

  return;


}//search_extract_pixel


void time_check(int Year, int Month, int Day, int Hour, int Minute, double LAT, double LNG)
{

  // Local Time
  int LTime[5+1];

  // Local Year/Month/Day/Hour/Minute/Second
  float *Time;

  // Filenama of Time Data
  char Timename[512];


  if((Time = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}

  sprintf(Timename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, Year, Month, Year, Month, Day, Hour, Minute, ttype);

  bin2f(Timename, X, Y, Time);

  jd_UTC_PickUP=TransDateToJD(Year,Month,Day,0,0,0);// 1.UTC Time to JD(UTC-UYYYY, UMM, UDD)  time_check(UYYYY, UMM, UDD, UHH, UMN, LAT, LNG);
  jd_UTC_PickUP = jd_UTC_PickUP +(double)Time[(Pick_Pix[2]-1)*X+Pick_Pix[1]]; // 2.Seek to JD(Pickup UTC time) 
  jd2ymdhn(jd_UTC_PickUP,Pick_Time_UTC); // 3.JD(UTC Pickup time)  to Calendar(Local Pickup time)         
  double jd_utctolocal = jd_UTC_PickUP + TIME_DIFF/24.0; // 4.JD(UTC Pickup time) to JD(Local) 
  jd2ymdhn(jd_utctolocal,Pick_Time_Local); // 5.JD(Local Pickup time)  to Calendar(Local Pickup time)        

  free(Time);

  return;

}//time_check


void shift_time(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, int *Shift_Time)
{

  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month-1;
  base.tm_mday = Day;
  base.tm_hour = Hour+Shift_HH;
  base.tm_min  = Minute;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Shift_Time[1] = shift->tm_year+1900;
  Shift_Time[2] = shift->tm_mon+1;
  Shift_Time[3] = shift->tm_mday;
  Shift_Time[4] = shift->tm_hour;
  Shift_Time[5] = shift->tm_min;


  return;


}//shift_time


int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;


  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month;
  base.tm_mday = 1;
  base.tm_hour = 0-1;
  base.tm_min  = 0;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Last_DD = shift->tm_mday;


  return Last_DD;


}//get_lastday


double date2doy(int Year, int Month, int Day, int Hour, int Minute)
{
  
    // Day of Year
    int DOY;
    
    time_t timer;
    struct tm base;
    struct tm *check;
    
    base.tm_year = Year-1900;
    base.tm_mon = Month-1;
    base.tm_mday = Day;
    base.tm_hour = Hour;
    base.tm_min  = Minute;
    base.tm_sec  = 0;
    
    timer = mktime(&base);
    
    check = localtime(&timer);
    
    DOY = check->tm_yday + 1;
   
    
    return DOY;
    
    
}//date2md


double cal_sza(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, double LAT, double LNG)
{
  
  // Day of Year
  int DOY;
  
  // Local Time
  int Local_Time[5+1];
  
  // Greenwith Meridian Time
  double UTC_Time;
  
  // Solar Declination
  double DEC;
  
  double A;
  double B;
  
  // Hour Angle
  double t;
  
  // Equqtion of Time
  double ET;
  
  // Mean Solar Time
  double MST;
  
  // True Solar Time
  double TST;
  
  // cos(Solar Zenith Angle)
  double SZA_cos;
  
  // Solar Zenith Angle
  double ANS;
  

  // Calculate HH + MN
  UTC_Time = Hour + (double)Minute / 60;
  
  // Calculate Local Time
  shift_time(Year, Month, Day, Hour, Minute, Shift_HH, Local_Time);
  
  // Calculate Day of Year
  DOY = date2doy(Local_Time[1], Local_Time[2], Local_Time[3], Local_Time[4], Local_Time[5]);
  
  LAT = LAT * M_PI / 180;
  
  
  A = (2*M_PI*(DOY-1)) / 365;
  DEC = b1 - b2 * cos(A) + b3 * sin(A) - b4 * cos(2*A) + b5 * sin(2*A) - b6 * cos(3*A) + b7 * sin(3*A);
  
  //printf("Solar Declination = %f\n", DEC);
  
  B =  (2*M_PI*(DOY-1)) / 365;
  ET = ((a1 + a2 * cos(B) - a3 * sin(B) - a4 * cos(2*B) - a5 * sin(2*B))*12) / M_PI;
  MST = UTC_Time + LNG / 15;
  TST = MST + ET;
  t = (15 * M_PI/180)*(TST-12);
  
  
  SZA_cos = cos(M_PI/2 - DEC)* cos(M_PI/2 - LAT) + sin(M_PI/2 - DEC) * sin(M_PI/2 - LAT) * cos(t);
  ANS = acos(SZA_cos) * 180.0/M_PI;
  
  //printf("%f\n", ANS);
  
  
  return ANS;
  
  
}//cal_sza


void date_errorcheck()
{

  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     printf("Year %d is less than or equal to 0\n", YYYY);
     exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     printf("Month %d does not exist\n", MM);
     exit(1);

  }//fi

  // Day Check
  Last_DD = get_lastday(YYYY, MM);

  if(DD > Last_DD || DD <= 0)
  {
     printf("Date of %d%02d%02d does not exist\n", YYYY, MM, DD);
     exit(1);

  }//fi


  return;


}//date_errorcheck


void initialization()
{

  int I,J;

  read_formatfile();


  if((STN_LAT = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL){ printf("malloc failed.\n"); exit(1);}
  if((STN_LNG = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL){ printf("malloc failed.\n"); exit(1);}

  // STN_LAT/LNG Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STN_LAT[(I-1)*X+J] = 0;
        STN_LNG[(I-1)*X+J] = 0;

     }//J
  }//I
 
  read_standard_latlng();

  return;


}//initialization


void read_formatfile()
{

  int I;

  // Present Line
  int pline = 1;

  char filename[512];
  char readline[1000];
  char *param[100+1];

  FILE *fp;


  sprintf(filename, "config/format.txt");


  for(I=0;I<=100;I++)
  {
     if((param[I] = (char *)malloc(sizeof(char)*1000)) == NULL){ printf("malloc failed.\n"); exit(1);}

  }


  // Read Format File
  printf("%s  Loading ...", filename);

  fp = fopen(filename, "r");

  if(fp == NULL)
  {

     printf("%s read error\n", filename);
     exit(1);

  }//fi


  while( fgets(readline, 1000, fp) != NULL)
  {
     strcpy(param[pline], strtok(readline, ", "));
     pline++;

  }//while

  printf(" finish\n");

  fclose(fp);

  TIME_DIFF = atof(param[1]);   // int->doble for India 5.5H
  sprintf(TIME_Code, "%s", param[2]);

  X = atoi(param[3]);
  Y = atoi(param[4]);

  sprintf(ftype, "%s", param[5]);
  sprintf(ttype, "%s", param[10]);

  sprintf(input_DIR, "%s", param[6]);
  sprintf(TimeS_DIR, "%s", param[7]);
  sprintf(SL_DIR, "config/");

  sprintf(LATname, "%s", param[8]);
  sprintf(LNGname, "%s", param[9]);

  Start_HH = 0;
  End_HH = 23;
  

  for(I=0;I<=100;I++) free(param[I]);


  return;


}//read_formatfile


void read_standard_latlng()
{

  // Filename of Latitude/Longitude Data
  char lat_filename[512];
  char lng_filename[512];

  sprintf(lat_filename, "%s%s", SL_DIR, LATname);
  sprintf(lng_filename, "%s%s", SL_DIR, LNGname);

  bin2f(lat_filename, X, Y, STN_LAT);
  bin2f(lng_filename, X, Y, STN_LNG);


  return;


}//read_standard_latlng


void write_txt(FILE *pw ) 
{
        fprintf(pw, "%7.4f %8.4f %6.3f %7.3f %9.4f %d %d %d %d %d %d %s \n", Extract_param[1], Extract_param[2], STN_LAT[(Pick_Pix[2]-1)*X+Pick_Pix[1]], STN_LNG[(Pick_Pix[2]-1)*X+Pick_Pix[1]], Extract_param[3], Pick_Time_Local[1], Pick_Time_Local[2], Pick_Time_Local[3], Pick_Time_Local[4], Pick_Time_Local[5], Pick_Time_Local[6], TIME_Code);
        return;

}//write_txt
